# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
from django.core.exceptions import ImproperlyConfigured
from rest_framework.test import APIRequestFactory
from elasticsearch_dsl import Search

from rest_framework_elasticsearch.es_filters import (
    ESFieldFilter, ElasticOrderingFilter, ElasticFieldsFilter,
    ElasticFieldsRangeFilter, ElasticSearchFilter)
from rest_framework_elasticsearch.es_views import ElasticAPIView
from .test_data import DataDocType, DATA
from .utils import get_search_ids


rf = APIRequestFactory()


class TestElasticAPIView:
    """ElasticAPIView tests class"""

    def create_view(self, es_client):
        """Create and return test view class instance

        Args:
            es_client (Elasticsearch): elasticsearch connection client
        Returns:
            ElasticAPIView: test view instance
        """
        view = ElasticAPIView()
        view.es_client = es_client
        view.es_model = DataDocType
        view.es_filter_backends = (
            ElasticSearchFilter,
            ElasticFieldsFilter,
            ElasticFieldsRangeFilter,
            ElasticOrderingFilter
        )
        return view

    def test_get_es_search_fields(self):
        view = ElasticAPIView()
        assert view.get_es_search_fields() is None

        es_search_fields = ('first_name', 'last_name')
        view.es_search_fields = es_search_fields
        assert view.get_es_search_fields() == es_search_fields

    def test_get_es_filter_fields(self):
        view = ElasticAPIView()
        assert view.get_es_filter_fields() == tuple()

        es_filter_fields = ('first_name', 'last_name')
        view.es_filter_fields = es_filter_fields
        assert view.get_es_filter_fields() == es_filter_fields

    def test_get_es_range_filter_fields(self):
        view = ElasticAPIView()
        assert view.get_es_range_filter_fields() == tuple()

        es_filter_fields = ('first_name', 'last_name')
        view.es_filter_fields = es_filter_fields
        assert view.get_es_range_filter_fields() == es_filter_fields

    def test_get_es_ordering_fields(self):
        view = ElasticAPIView()
        assert view.get_es_ordering_fields() is None

        es_ordering_fields = ('first_name', 'last_name')
        view.es_ordering_fields = es_ordering_fields
        assert view.get_es_ordering_fields() == es_ordering_fields

    def test_get_es_excludes_fields(self):
        view = ElasticAPIView()
        assert view.get_es_excludes_fields() is None

        es_excludes_fields = ('first_name', 'last_name')
        view.es_excludes_fields = es_excludes_fields
        assert view.get_es_excludes_fields() == es_excludes_fields

    def test_get_es_client_without_es_client(self):
        view = ElasticAPIView()
        with pytest.raises(Exception) as err:
            view.get_es_client()
            assert err.message == (
                "'%s' should either include a `queryset` attribute, "
                "or override the `get_queryset()` method."
                % view.__class__.__name__
            )

    def test_get_es_client_with_wrong_es_client(self):
        view = ElasticAPIView()
        view.es_client = str
        with pytest.raises(ValueError):
            view.get_es_client()

    def test_get_es_client(self, es_data_client):
        view = ElasticAPIView()
        view.es_client = es_data_client
        assert view.get_es_client() == es_data_client

    def test_get_es_search_without_es_model(self, es_data_client):
        view = ElasticAPIView()
        with pytest.raises(ImproperlyConfigured) as err:
            view.get_es_search()
            assert err.message == (
                "Cannot use %s on a view which does not have the 'es_model'" %
                view.__class__.__name__
            )

    def test_get_es_search(self, es_data_client):
        view = self.create_view(es_data_client)
        expected = Search(using=es_data_client,
                          index='test',
                          doc_type=DataDocType)
        assert view.get_es_search().to_dict() == expected.to_dict()

    @pytest.mark.parametrize('query_params, expected', [
        (
            {'search': 'Ford Prefect'},
            [2, 8, 10, 5, 6, 12]
        ),
        (
            {'search': 'Earth'},
            [5, 3, 14]
        ),
        (
            {'search': 'Ford Prefect', 'active': 'False'},
            [6, 8, 10]
        ),
        (
            {'search': 'Ford Prefect', 'from_score': '200'},
            [2, 6, 8, 10]
        ),
    ])
    def test_filter_search(self, search, es_client, query_params, expected):
        view = self.create_view(es_client)
        view.es_search_fields = ('description',)
        view.es_filter_fields = (
            ESFieldFilter('skills'),
            ESFieldFilter('score'),
            ESFieldFilter('active', 'is_active')
        )

        view.request = rf.get('/test/')
        view.request.query_params = query_params

        search = view.filter_search(search)
        result = get_search_ids(search)
        assert sorted(result) == sorted(expected)

    def test_es_representation(self, search, es_client):
        view = self.create_view(es_client)

        view.request = rf.get('/test/')
        view.request.query_params = {'active': 'False'}

        search = view.filter_search(search)
        result = search[:len(DATA)].execute()
        expected = [item.to_dict() for item in result]
        assert view.es_representation(result) == expected

    def test_excludes_respond_fields(self, search, es_client):
        view = self.create_view(es_client)
        view.es_excludes_fields = (
            'last_name',
            'city',
            'skills',
            'birthday',
            'is_active',
            'description',
            'score'
        )

        view.request = rf.get('/test/')
        view.request.query_params = {'active': 'False'}

        search = view.excludes_respond_fields(search)
        result = view.es_representation(search[:len(DATA)].execute())
        expected = {'first_name': 'test'}.keys()
        for item in result:
            assert item.keys() == expected

    def test_do_search(self, search, es_client):
        view = self.create_view(es_client)
        # mock get_es_search method
        view.get_es_search = lambda: search
        view.es_search_fields = ('description',)
        view.es_excludes_fields = ('birthday', 'description', 'score')

        view.request = rf.get('/test/')
        view.request.query_params = {'search': 'Earth', 'active': 'False'}

        search = view.do_search()
        result = search[:len(DATA)].execute()
        expected = [item.to_dict() for item in result]
        assert view.es_representation(result) == expected

    def test_get_queryset(self, search, es_client):
        view = self.create_view(es_client)
        # mock get_es_search method
        view.get_es_search = lambda: search
        view.es_filter_fields = (
            ESFieldFilter('active', 'is_active'),
        )
        view.es_search_fields = ('description',)
        view.es_ordering_fields = ("first_name",)
        view.es_excludes_fields = (
            'birthday',
            'score',
            'skills',
            'description'
        )

        view.request = rf.get('/test/')
        view.request.query_params = {
            'search': 'Earth',
            'active': 'False',
            'ordering': 'first_name'
        }

        result = view.get_queryset()
        assert result == [
            {
                'city': 'Manila',
                'first_name': 'Samantha',
                'is_active': False,
                'last_name': 'Arundhati'
            }
        ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .test_data import DATA


def get_search_ids(search):
    """Execute search and return list of items ids

    Args:
        search (elasticsearch_dsl.Search): search instance
    Returns:
        [int]: list of items ids
    """
    return [int(item.meta.id) for item in search[:len(DATA)].execute()]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
from rest_framework.test import APIRequestFactory

from rest_framework_elasticsearch.es_mixins import ListElasticMixin
from rest_framework_elasticsearch.es_views import ElasticAPIView
from rest_framework_elasticsearch.es_pagination import (
    ElasticLimitOffsetPagination)
from .test_data import DATA, DataDocType


rf = APIRequestFactory()


def create_view(es_client):
    """Create and return test view class instance

    Args:
        es_client (Elasticsearch): elasticsearch connection client
    Returns:
        (ListElasticMixin, ElasticAPIView): test view instance
    """

    class View(ListElasticMixin, ElasticAPIView):
        es_pagination_class = ElasticLimitOffsetPagination
        es_model = DataDocType
        es_search_fields = ('first_name',)

    return View()


def test_es_paginator_none():
    view = ListElasticMixin()
    assert view.es_paginator is None
    assert view._es_paginator is None


def test_es_paginator_class():
    view = ListElasticMixin()
    view.es_pagination_class = ElasticLimitOffsetPagination
    assert isinstance(view.es_paginator, ElasticLimitOffsetPagination)
    assert isinstance(view._es_paginator, ElasticLimitOffsetPagination)


def test_paginate_search_none(search):
    view = ListElasticMixin()
    assert view.paginate_search(search) is None


def test_get_paginated_response_none(search):
    view = ListElasticMixin()
    with pytest.raises(Exception):
        view.get_paginated_response(search)


def test_get_paginated_response(search):
    view = ListElasticMixin()
    view.es_pagination_class = ElasticLimitOffsetPagination

    view.request = rf.get('/test/')
    view.request.query_params = {}

    page = view.paginate_search(search)
    response = view.get_paginated_response(page)

    assert response.status_code == 200
    assert response.data['count'] == len(DATA)
    assert 'next' in response.data
    assert 'previous' in response.data
    assert 'results' in response.data


def test_list_paginated(search, es_client):
    view = create_view(es_client)
    # mock get_es_search method
    view.get_es_search = lambda: search

    view.request = rf.get('/test/')
    view.request.query_params = {}

    response = view.list(view.request)

    assert response.status_code == 200
    assert response.data['count'] == len(DATA)
    assert 'next' in response.data
    assert 'previous' in response.data
    assert len(response.data['results']) == 10


def test_list_representation(search, es_client):
    view = create_view(es_client)
    # mock get_es_search method
    view.get_es_search = lambda: search

    view.request = rf.get('/test/')
    view.request.query_params = {'search': 'Zofia'}

    response = view.list(view.request)

    assert response.status_code == 200
    assert response.data['count'] == 1
    assert response.data['next'] is None
    assert response.data['previous'] is None
    assert len(response.data['results']) == 1



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os

import pytest

from elasticsearch_dsl import search as search_
from elasticsearch_dsl.connections import connections
from elasticsearch.helpers import bulk
from elasticsearch import Elasticsearch

from .test_data import create_test_index, DATA


def pytest_configure():
    import django
    from django.conf import settings

    settings.configure(
        SITE_ID=1,
        SECRET_KEY='not very secret in tests',
        ROOT_URLCONF='tests.urls',
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'APP_DIRS': True,
                'OPTIONS': {
                    "debug": True,  # We want template errors to raise
                }
            },
        ],
        ALLOWED_HOSTS=['localhost', 'testserver']
    )

    django.setup()


@pytest.fixture(scope='session')
def es_client():
    """Create and return elasticsearch connection"""
    connection = Elasticsearch([os.environ.get('TEST_ES_SERVER', {})])
    connections.add_connection('default', connection)
    return connection


@pytest.fixture(scope='function')
def es_data_client(es_client):
    """Create elasticsearch index with preparing data and
    delete after exit of the scope.
    """
    create_test_index()
    bulk(es_client, DATA, raise_on_error=True, refresh=True)
    yield es_client
    es_client.indices.delete('test-*', ignore=404)


@pytest.fixture
def search(es_data_client, scope='function'):
    """Create and return Search test instance"""
    return search_.Search(index='test')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
from rest_framework.test import APIRequestFactory
from elasticsearch_dsl import Q

from rest_framework_elasticsearch.es_filters import (
    ESFieldFilter, ElasticOrderingFilter, ElasticFieldsFilter,
    ElasticFieldsRangeFilter, ElasticSearchFilter)
from rest_framework_elasticsearch.es_views import ElasticAPIView
from .test_data import DataDocType, DATA
from .utils import get_search_ids


rf = APIRequestFactory()


@pytest.mark.parametrize('dataset, expected', [
    (
        ('label', 'name', 'description'),
        ('label', 'name', 'description')
    ),
    (
        ('label', None, 'description'),
        ('label', 'label', 'description')
    ),
    (
        ('label', None, None),
        ('label', 'label', None)
    )
])
def test_es_field_filters(dataset, expected):
    ffilter = ESFieldFilter(dataset[0],
                            name=dataset[1],
                            description=dataset[2])
    assert expected == (ffilter.label, ffilter.name, ffilter.description)


class TestElasticOrderingFilter:

    def setup_method(self):
        self.backend = ElasticOrderingFilter()

    def create_view(self, es_ordering_fields):
        """Create and return test view class instance

        Args:
            es_ordering_fields (tuple): ordering fields
        Returns:
            ElasticAPIView: test view instance
        """
        view = ElasticAPIView()
        view.es_ordering_fields = es_ordering_fields
        return view

    @pytest.mark.parametrize('es_ordering_fields, expected', [
        (
            ("first_name", ("is_active", "active")),
            ("first_name", ("is_active", "active")),
        ),
        (
            "first_name",
            ("first_name",)
        )
    ])
    def test_get_es_ordering_fields(self, es_ordering_fields, expected):
        view = self.create_view(es_ordering_fields)
        result = self.backend.get_es_ordering_fields(view)
        assert expected == result

    @pytest.mark.parametrize('es_ordering_fields, expected', [
        (
            ("is_active", "active"),
            ("is_active", "active")
        ),
        (
            "first_name",
            ("first_name", "first_name")
        )
    ])
    def test_validation(self, es_ordering_fields, expected):
        result = ElasticOrderingFilter.validation(es_ordering_fields)
        assert expected == result

    def test_get_valid_fields_with_es_ordering_fields(self):
        es_ordering_fields = (
            "first_name",
            "last_name",
            ("is_active", "active")
        )
        view = self.create_view(es_ordering_fields)
        expected = [
            ("first_name", "first_name"),
            ("last_name", "last_name"),
            ("is_active", "active")
        ]
        result = self.backend.get_valid_fields([], view)
        assert result == expected

    def test_get_valid_fields_without_es_ordering_fields(self):
        view = self.create_view(None)
        valid_fields = []
        self.backend.get_default_valid_fields = lambda q, v: valid_fields
        result = self.backend.get_valid_fields([], view)
        assert result == valid_fields

    @pytest.mark.parametrize('fields, expected', [
        (
            ['first_name', 'last_name', '-active'],
            ['first_name', '-is_active']
        ),
        (
            ['+first_name', 'last_name', '#active'],
            ['+first_name']
        ),
        (
            ['+first_name', '-active'],
            ['+first_name', '-is_active']
        )
    ])
    def test_remove_invalid_fields(self, fields, expected):
        es_ordering_fields = (
            "first_name",
            ("is_active", "active")
        )
        view = self.create_view(es_ordering_fields)
        request = rf.get('/test/')
        result = self.backend.remove_invalid_fields([], fields, view, request)
        assert result == expected

    def test_filter_search(self, search):

        def get_expected():
            """Return expected data items sorted by id"""
            items = [
                (
                    item['_id'],
                    item['_source']['first_name'],
                    item['_source']['is_active']
                )
                for item in DATA
            ]
            items = sorted(items, key=lambda tup: (tup[1], not tup[2]))
            return items

        es_ordering_fields = (
            ("first_name", "first_name"),
            ("is_active", "-active")
        )
        view = self.create_view(es_ordering_fields)

        request = rf.get('/test/')
        request.query_params = {'ordering': 'first_name,active'}

        search = self.backend.filter_search(request, search, view)
        result = [
            (item.meta.id, item.first_name, item.is_active)
            for item in search[:len(DATA)].execute()
        ]
        assert result == get_expected()


class TestElasticFieldsFilter:

    def setup_method(self):
        self.backend = ElasticFieldsFilter()

    def create_view(self, es_filter_fields):
        """Create and return test view class instance

        Args:
            es_filter_fields ([ESFieldFilter]): filtering fields
        Returns:
            ElasticAPIView: test view instance
        """
        view = ElasticAPIView()
        view.es_model = DataDocType
        view.es_filter_fields = es_filter_fields
        return view

    def test_get_es_filter_fields(self):
        es_filter_fields = (
            ESFieldFilter('skills'),
            ESFieldFilter('active', 'is_active')
        )
        view = self.create_view(es_filter_fields)
        result = self.backend.get_es_filter_fields(view)
        assert result == es_filter_fields

    @pytest.mark.parametrize('es_filter_fields, query_params, expected', [
        (
            [ESFieldFilter('active', 'is_active')],
            {'active': 'False'},
            [3, 6, 8, 10]
        ),
        (
            [
                ESFieldFilter('birthday')
            ],
            {'birthday': '1985-03-17T12:20:09'},
            [1]
        ),
        (
            [ESFieldFilter('skills')],
            {'skills': 'python'},
            [1, 4, 5, 10]
        ),
        (
            [ESFieldFilter('skills')],
            {'skills': 'python,ruby'},
            [1, 4, 5, 6, 10]
        ),
        (
            [ESFieldFilter('active', 'is_active')],
            {'active': 'False', 'skills': 'python'},
            [3, 6, 8, 10]
        ),
        (
            [ESFieldFilter('active', 'is_active')],
            {'active': 'False', 'skills': 'python'},
            [3, 6, 8, 10]
        ),
        (
            [ESFieldFilter('score')],
            {'score': '200'},
            [2, 13]
        ),
    ])
    def test_filter_search(self, search, es_filter_fields,
                           query_params, expected):
        view = self.create_view(es_filter_fields)
        request = rf.get('/test/')
        request.query_params = query_params
        search = self.backend.filter_search(request, search, view)
        result = get_search_ids(search)
        assert sorted(result) == sorted(expected)


class TestElasticFieldsRangeFilter:

    def setup_method(self):
        self.backend = ElasticFieldsRangeFilter()

    def create_view(self, es_range_filter_fields):
        """Create and return test view class instance

        Args:
            es_range_filter_fields ([ESFieldFilter]): filtering range fields
        Returns:
            ElasticAPIView: test view instance
        """
        view = ElasticAPIView()
        view.es_model = DataDocType
        view.es_filter_fields = es_range_filter_fields
        return view

    def test_get_es_filter_fields(self):
        es_range_filter_fields = (
            ESFieldFilter('skills'),
            ESFieldFilter('active', 'is_active')
        )
        view = self.create_view(es_range_filter_fields)
        result = self.backend.get_es_filter_fields(view)
        assert result == es_range_filter_fields

    @pytest.mark.parametrize('es_filter_fields, query_params, expected', [
        (
            [ESFieldFilter('score')],
            {'from_score': '500'},
            [6, 7, 8, 10, 11]
        ),
        (
            [ESFieldFilter('score')],
            {'to_score': '100'},
            [1, 3, 5, 9, 12, 14]
        ),
        (
            [ESFieldFilter('score')],
            {'from_score': '500', 'to_score': '600'},
            [7, 8, 10]
        ),
        (
            [ESFieldFilter('score')], {}, [int(item['_id']) for item in DATA]
        ),
    ])
    def test_filter_search(self, search, es_filter_fields,
                           query_params, expected):
        view = self.create_view(es_filter_fields)
        request = rf.get('/test/')
        request.query_params = query_params
        search = self.backend.filter_search(request, search, view)
        result = get_search_ids(search)
        assert sorted(result) == sorted(expected)


class TestElasticSearchFilter:

    def setup_method(self):
        self.backend = ElasticSearchFilter()

    def create_view(self, es_search_fields):
        """Create and return test view class instance

        Args:
            es_search_fields ([ESFieldFilter]): search fields
        Returns:
            ElasticAPIView: test view instance
        """
        view = ElasticAPIView()
        view.es_model = DataDocType
        view.es_search_fields = es_search_fields
        return view

    @pytest.mark.parametrize('search_param, query_params, expected', [
        (
            None, {'search': 'test'}, 'test'
        ),
        (
            'search', {'search': 'test'}, 'test'
        ),
        (
            'q', {'q': 'test'}, 'test'
        ),
        (
            'search', {'q': 'test'}, ''
        ),
    ])
    def test_get_search_query(self, search_param, query_params, expected):
        request = rf.get('/test/')
        request.query_params = query_params
        if search_param:
            self.backend.search_param = search_param
        result = self.backend.get_search_query(request)
        assert result == expected

    def test_get_es_query(self):

        class TestElasticSearchFilter(ElasticSearchFilter):
            def get_es_query(self, s_query, s_fields, **kwargs):
                return Q("match", query=s_query, field=s_fields)

        s_query = "test"
        s_fields = "first_name"
        backend = TestElasticSearchFilter()
        result = backend.get_es_query(s_query, s_fields)
        expected = Q("match", query=s_query, field=s_fields)
        assert result == expected

    @pytest.mark.parametrize('es_search_fields, query_params, expected', [
        (
            ('first_name',),
            {'search': 'Zofia'},
            [1]
        ),
        (
            ('first_name', 'last_name', 'city'),
            {'search': 'Zofia'},
            [1]
        ),
        (
            ('first_name', 'last_name', 'city'),
            {'search': 'Zofia Rome'},
            [4, 7]
        ),
        (
            ('description'),
            {'search': 'information'},
            [2]
        ),
        (
            ('description'),
            {'search': 'Ford Prefect'},
            [2, 8, 10, 5, 6, 12]
        ),
        (
            ('description'),
            {'search': 'Earth'},
            [5, 3, 14]
        ),
        (
            ('description'),
            {'search': 'The Hitchhiker’s Guide'},
            [8]
        ),
    ])
    def test_filter_search(self, search, es_search_fields,
                           query_params, expected):
        view = self.create_view(es_search_fields)
        request = rf.get('/test/')
        request.query_params = query_params
        search = self.backend.filter_search(request, search, view)
        result = get_search_ids(search)
        assert result == expected



# -*- coding: utf-8 -*-
# http://www.django-rest-framework.org/topics/3.7-announcement/#customizing-api-docs-schema-generation
try:
    from rest_framework.schemas import AutoSchema
except ImportError:
    EsAutoSchema = type("EsAutoSchema", tuple(), dict())
else:
    from rest_framework.schemas.utils import is_list_view

    class EsAutoSchema(AutoSchema):
        """ Elasticsearch inspector for APIView

        Responsible for per-view instrospection and schema generation.
        """
        def get_es_filter_fields(self, path, method):
            fields = []
            for filter_backend in self.view.es_filter_backends:
                fields += filter_backend().get_schema_fields(self.view)
            return fields

        def get_filter_fields(self, path, method):
            fields = super(EsAutoSchema, self).get_filter_fields(path, method)
            fields += self.get_es_filter_fields(path, method)
            return fields

        def get_es_pagination_fields(self, path, method):
            view = self.view
            if not is_list_view(path, method, view):
                return []

            pagination = getattr(view, 'es_pagination_class', None)
            if not pagination:
                return []

            return pagination().get_schema_fields(view)

        def get_pagination_fields(self, path, method):
            fields = super(EsAutoSchema, self).get_pagination_fields(path, method)
            fields += self.get_es_pagination_fields(path, method)
            return fields



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import copy

import pytest
from django.db import models

from rest_framework_elasticsearch.es_serializer import (
    BaseElasticSerializer, ElasticSerializer, ElasticModelSerializer)
from .test_data import DataDocType, DATA


class DjangoModel(models.Model):
    """Django ORM test model"""
    first_name = models.CharField(max_length=100)

    class Meta:
        app_label = 'tests'


class TestBaseElasticSerializer:

    def setup_method(self):
        self.serializer = BaseElasticSerializer()
        # mock serializer Meta class
        self.serializer.Meta = type(str('Meta'), tuple(), {})

    def test_es_instance(self):
        with pytest.raises(NotImplementedError):
            self.serializer.es_instance()

    def test_get_es_model_without_es_model(self):
        with pytest.raises(ValueError) as err:
            self.serializer.get_es_model()
            assert err.message == 'Can not find es_model value'

    def test_get_es_model(self):
        self.serializer.Meta.es_model = DataDocType
        assert self.serializer.get_es_model() == DataDocType

    def test_save_and_delete(self, es_client):
        # Preparing test data
        self.serializer.Meta.es_model = DataDocType
        instance = DataDocType(**DATA[0]['_source'])
        self.serializer.es_instance = lambda: instance

        # Test save method
        self.serializer.save()
        instance_id = instance.meta['id']
        result = DataDocType.get(id=instance_id)
        assert instance.to_dict() == result.to_dict()

        # Test Delete method
        self.serializer.delete()
        result = DataDocType.get(id=instance_id, ignore=404)
        assert result is None


class TestElasticSerializer:

    def setup_method(self):
        self.serializer = ElasticSerializer()
        # mock serializer Meta class
        self.serializer.Meta = type(str('Meta'), tuple(), {})
        self.serializer.Meta.es_model = DataDocType

    def test_get_es_instace_pk(self):
        # Test without id value
        with pytest.raises(ValueError) as err:
            self.serializer.get_es_instace_pk({})
            assert err.message == 'Can not save object without id'

        # Test with id value
        assert self.serializer.get_es_instace_pk({'id': 1}) == 1

    def test_es_repr(self):
        instance = DataDocType(**DATA[0]['_source'])
        data = instance.to_dict()
        data['id'] = 1
        model = self.serializer.es_repr(copy.copy(data))
        assert model.to_dict() == data


class TestElasticModelSerializer:

    def setup_method(self):
        self.serializer = ElasticModelSerializer()
        # mock serializer Meta class
        self.serializer.Meta = type(str('Meta'), tuple(), {})
        self.serializer.Meta.es_model = DataDocType

    def test_get_es_instace_pk(self):
        instance = DjangoModel(first_name='test')
        instance.pk = 1
        assert self.serializer.get_es_instace_pk(instance) == 1

    def test_es_repr(self):
        self.serializer.Meta.model = DjangoModel
        self.serializer.Meta.fields = ['first_name']

        instance = DjangoModel(first_name='test')
        instance.pk = 1

        model = self.serializer.es_repr(instance)
        assert model.to_dict() == {'first_name': 'test'}



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import elasticsearch_dsl as es


class DataDocType(es.DocType):
    """Elasticsearch test model"""
    first_name = es.Keyword()
    last_name = es.Keyword()
    city = es.Text()
    skills = es.Keyword()
    birthday = es.Date()
    is_active = es.Boolean()
    score = es.Integer()
    description = es.Text()

    class Meta:
        index = 'test'


def create_test_index():
    """Initialize test index"""
    DataDocType.init()


DATA = [
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '1',
        '_source': {
            'first_name': 'Zofia',
            'last_name': 'Marie',
            'city': 'Warsaw',
            'skills': ['python', 'sql', 'js'],
            'birthday': '1985-03-17T12:20:09',
            'is_active': True,
            'score': 100,
            'description': """
                He had made one careless blunder though, because he had skimped
                a bit on his preparatory research.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '2',
        '_source': {
            'first_name': 'Callisto',
            'last_name': 'Alexandra',
            'city': 'San Jose',
            'skills': ['java', 'c#', 'sql'],
            'birthday': '1990-02-07T15:22:09',
            'is_active': True,
            'score': 200,
            'description': """
                The information he had gathered had led him to choose the name
                Ford Prefect as being nicely inconspicuous.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '3',
        '_source': {
            'first_name': 'Samantha',
            'last_name': 'Arundhati',
            'city': 'Manila',
            'skills': ['html', 'js', 'css'],
            'birthday': '1987-05-27T16:08:02',
            'is_active': False,
            'score': 100,
            'description': """
                He struck most of the friends he had made on Earth as an
                eccentric, but a harmless one – an unruly boozer with some
                oddish habits.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '4',
        '_source': {
            'first_name': 'Crocetta',
            'last_name': 'Florette',
            'city': 'Rome',
            'skills': ['python', 'go', 'sql', 'django'],
            'birthday': '1991-04-07T16:38:29',
            'is_active': True,
            'score': 450,
            'description': """
                Thereafter, staggering semi-paralytic down the night streets he
                would of ten ask passing policemen if they knew the way to
                Betelgeuse.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '5',
        '_source': {
            'first_name': 'Akhmad',
            'last_name': 'Farouk',
            'city': 'Ottawa',
            'skills': ['python', 'flask', 'django'],
            'birthday': '1992-01-21T12:18:09',
            'is_active': True,
            'score': 10,
            'description': """
                Ford Prefect was desperate that any flying saucer at all would
                arrive soon because fifteen years was a long time to get
                stranded anywhere, particularly somewhere as mindboggingly dull
                as the Earth.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '6',
        '_source': {
            'first_name': 'Rohan',
            'last_name': 'Dand',
            'city': 'Ottawa',
            'skills': ['ruby', 'ror', 'sql'],
            'birthday': '1991-09-07T11:20:29',
            'is_active': False,
            'score': 1000,
            'description': """
                Ford wished that a flying saucer would arrive soon because he
                knew how to flag flying saucers down and get lifts from them.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '7',
        '_source': {
            'first_name': 'Odell',
            'last_name': 'Leonidas',
            'city': 'Rome',
            'skills': ['angular', 'react', 'css'],
            'birthday': '1993-09-20T16:20:08',
            'is_active': True,
            'score': 500,
            'description': """
                He knew how to see the Marvels of the Universe for less than
                thirty Altairan dollars a day.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '8',
        '_source': {
            'first_name': 'Markus',
            'last_name': 'Emmeline',
            'city': 'Ottawa',
            'skills': ['php', 'html'],
            'birthday': '1991-04-26T16:48:03',
            'is_active': False,
            'score': 600,
            'description': """
                In fact, Ford Prefect was a roving researcher for that wholly
                remarkable book The Hitchhiker’s Guide to the Galaxy.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '9',
        '_source': {
            'first_name': 'Gabriel',
            'last_name': 'Sara',
            'city': 'Baku',
            'skills': ['js', 'css', 'html', 'sql'],
            'birthday': '1988-08-27T16:18:29',
            'is_active': True,
            'score': 100,
            'description': """
                Human beings are great adaptors, and by lunchtime life in the
                environs of Arthur’s house had settled into a steady routine.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '10',
        '_source': {
            'first_name': 'Merilyn',
            'last_name': 'Barrie',
            'city': 'Rabat',
            'skills': ['python', 'sql'],
            'birthday': '1992-03-11T11:28:09',
            'is_active': False,
            'score': 600,
            'description': """
                Arthur looked up and squinting into the sun was startled to see
                Ford Prefect standing above him.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '11',
        '_source': {
            'first_name': 'Sunil',
            'last_name': 'Allan',
            'city': 'Lima',
            'skills': ['java', 'android'],
            'birthday': '1991-05-20T17:28:32',
            'is_active': True,
            'score': 900,
            'description': """
                Then suddenly he squatted down beside Arthur.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '12',
        '_source': {
            'first_name': 'Navin',
            'last_name': 'Mladen',
            'city': 'Doha',
            'skills': ['swift', 'c'],
            'birthday': '1990-04-23T16:48:09',
            'is_active': True,
            'score': 10,
            'description': """
                Ford stared at Arthur, and Arthur was astonished to find that
                his will was beginning to weaken.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '13',
        '_source': {
            'first_name': 'Helga',
            'last_name': 'Terell',
            'city': 'Oslo',
            'skills': ['c++', 'c#'],
            'birthday': '1989-01-20T06:18:02',
            'is_active': True,
            'score': 200,
            'description': """
                Two contestants would sit either side of a table, with a glass
                in front of each of them.
            """
        },
    },
    {
        '_index': 'test',
        '_type': 'doc',
        '_id': '14',
        '_source': {
            'first_name': 'Lynsey',
            'last_name': 'Brynja',
            'city': 'Oslo',
            'skills': ['c++', 'java'],
            'birthday': '1984-07-21T12:18:29',
            'is_active': True,
            'score': 100,
            'description': """
                The game was not unlike the Earth game called Indian Wrestling,
                and was played like this.
            """
        },
    },
]



# -*- coding: utf-8 -*-
from abc import ABCMeta, abstractmethod

import six


@six.add_metaclass(ABCMeta)
class BaseESFieldValidator:
    # Elastycsearch field types
    es_types = []

    @staticmethod
    @abstractmethod
    def validate(value):
        # must be implemented in your class, this method
        # validated value and retun python type or None
        pass


class BooleanFieldValidator(BaseESFieldValidator):
    es_types = ['boolean']

    @staticmethod
    def validate(value):
        if value in (True, 'True', 'true', '1', 1):
            return True
        elif value in (False, 'False', 'false', '0', 0):
            return False
        return None


class IntegerFieldValidator(BaseESFieldValidator):
    es_types = ['short', 'integer', 'long']

    @staticmethod
    def validate(value):
        try:
            data = int(value)
        except (ValueError, TypeError):
            data = None
        return data


class FloatFieldValidator(BaseESFieldValidator):
    es_types = ['float', 'double']

    @staticmethod
    def validate(value):
        try:
            data = float(value)
        except (ValueError, TypeError):
            data = None
        return data


class ESFieldValidator:
    """ Factory class for Elastycsearch field validators.

    Methood `clean_field` accepts two position arguments - field_type and value,
    get validators and return converted python value or None.
    """
    validators = (
        BooleanFieldValidator,
        IntegerFieldValidator,
        FloatFieldValidator
    )

    def __init__(self, *args, **kwargs):
        self._validators = {t: v for v in self.validators for t in v.es_types}

    def validate(self, field_type, value):
        validator = self._validators.get(field_type)
        return validator.validate(value) if validator else value


field_validator = ESFieldValidator()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

from rest_framework.test import APIRequestFactory
from rest_framework_elasticsearch.es_pagination import (
    ElasticLimitOffsetPagination)
from .test_data import DATA


rf = APIRequestFactory()


@pytest.fixture
def paginator(scope='function'):
    return ElasticLimitOffsetPagination()


def test_paginate_get_count(search, paginator):
    result = paginator._get_count(search)
    assert result == len(DATA)


@pytest.mark.parametrize('limit, offset, expected', [
    (5, 0, 5),
    (5, 5, 5),
    (5, 10, 4),
    (100, 0, 14),
    (100, 6, 8),
    (100, 50, 0),
    # default limit value is 10
    (0, 0, 10),
])
def test_paginate_search(search, paginator, limit, offset, expected):
    request = rf.get('/test/')
    request.query_params = {'limit': limit, 'offset': offset}
    result = paginator.paginate_search(search, request)
    assert len(result) == expected



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function, unicode_literals)

from rest_framework.pagination import LimitOffsetPagination


class ElasticLimitOffsetPagination(LimitOffsetPagination):
    default_limit = 10

    def _get_count(self, search):
        response = search.execute()
        return response.hits.total

    def paginate_search(self, search, request, view=None):
        """
        Paginate a queryset if required, either returning a
        page object, or `None` if pagination is not configured for this view.
        """
        self.limit = self.get_limit(request)
        if self.limit is None:
            return None
        self.offset = self.get_offset(request)
        self.count = self._get_count(search)
        self.request = request
        if self.count > self.limit and self.template is not None:
            self.display_page_controls = True

        if self.count == 0 or self.offset > self.count:
            return []
        return list(search[self.offset:self.offset + self.limit])



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from rest_framework import serializers


class BaseElasticSerializer(object):
    def es_instance(self):
        raise NotImplementedError

    def get_es_model(self):
        if not hasattr(self.Meta, 'es_model'):
            raise ValueError(
                'Can not find es_model value'
            )
        return self.Meta.es_model

    def save(self, using=None, index=None, validate=True, **kwargs):
        instance = self.es_instance()
        instance.save(using=using, index=index, validate=validate, **kwargs)

    def delete(self, using=None, index=None, **kwargs):
        instance = self.es_instance()
        instance.delete(using=using, index=index, **kwargs)


class ElasticSerializer(BaseElasticSerializer,
                        serializers.Serializer):
    def get_es_instace_pk(self, data):
        try:
            return data['id']
        except KeyError:
            raise ValueError(
                'Can not save object without id'
            )

    def es_repr(self, data):
        data['meta'] = dict(id=self.get_es_instace_pk(data))
        model = self.get_es_model()
        return model(**data)

    def es_instance(self):
        if not self.is_valid():
            raise serializers.ValidationError(self.errors)
        return self.es_repr(self.data)


class ElasticModelSerializer(BaseElasticSerializer,
                             serializers.ModelSerializer):
    def get_es_instace_pk(self, instance):
        return instance.pk

    def es_repr(self, instance):
        data = self.to_representation(instance)
        data['meta'] = dict(id=self.get_es_instace_pk(instance))
        model = self.get_es_model()
        return model(**data)

    def es_instance(self):
        if not self.instance:
            raise ValueError("Can't reproduce object")
        return self.es_repr(self.instance)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from rest_framework.response import Response


class ListElasticMixin(object):
    es_pagination_class = None

    @property
    def es_paginator(self):
        """The es_paginator instance associated with the view."""
        if not hasattr(self, '_es_paginator'):
            if self.es_pagination_class is None:
                self._es_paginator = None
            else:
                self._es_paginator = self.es_pagination_class()
        return self._es_paginator

    def paginate_search(self, search):
        """
        Return a single page of results, or `None` if pagination is disabled.
        """
        if self.es_paginator is None:
            return None
        return self.es_paginator.paginate_search(search, self.request, view=self)

    def get_paginated_response(self, data):
        """
        Return a paginated style `Response` object for the given output data.
        """
        assert self.es_paginator is not None
        return self.es_paginator.get_paginated_response(data)

    def list(self, request, *args, **kwargs):
        search = self.do_search()

        page = self.paginate_search(search)
        if page is not None:
            return self.get_paginated_response(self.es_representation(page))

        return Response(self.es_representation(search.scan()))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest

from rest_framework_elasticsearch import es_validators


@pytest.mark.parametrize("value", (True, 'True', 'true', '1'))
def test_boolean_validator_true(value):
    assert es_validators.BooleanFieldValidator.validate(value)


@pytest.mark.parametrize("value", (False, 'False', 'false', '0'))
def test_boolean_validator_false(value):
    assert not es_validators.BooleanFieldValidator.validate(value)


@pytest.mark.parametrize("value", (20, 1.5, [5], {}, 'test', None))
def test_boolean_validator_invalid(value):
    assert es_validators.BooleanFieldValidator.validate(value) is None


@pytest.mark.parametrize("value", (0, 1, 50, 2.5, 300.02, 10e9, '42'))
def test_integer_validator_valid(value):
    expected = int(value)
    assert es_validators.IntegerFieldValidator.validate(value) == expected


@pytest.mark.parametrize("value", ('one', 'a1', {}, [], None))
def test_integer_validator_invalid(value):
    assert es_validators.IntegerFieldValidator.validate(value) is None


@pytest.mark.parametrize("value", (0, 1, 50, 2.5, 300.02, 10e9, '42'))
def test_float_validator_valid(value):
    expected = float(value)
    assert es_validators.FloatFieldValidator.validate(value) == expected


@pytest.mark.parametrize("value", ('one', 'a1', {}, [], None))
def test_float_validator_invalid(value):
    assert es_validators.FloatFieldValidator.validate(value) is None


@pytest.mark.parametrize("field_type, value, expected", [
    # BooleanFieldValidator
    ('boolean', 'true', True),
    ('boolean', '0', False),
    # IntegerFieldValidator
    ('short', '1', 1),
    ('integer', '100', 100),
    ('long', 2.5, 2),
    # FloatFieldValidator
    ('float', '2.5', 2.5),
    ('double', '100.01', 100.01),
    # Wrong field type
    ('test', 'test', 'test'),
])
def test_es_field_validator(field_type, value, expected):
    validator = es_validators.ESFieldValidator()
    assert validator.validate(field_type, value) == expected



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.exceptions import ImproperlyConfigured
from elasticsearch import Elasticsearch
from elasticsearch_dsl import Search
from rest_framework import views

from .es_filters import ElasticSearchFilter
from .es_inspector import EsAutoSchema
from .es_mixins import ListElasticMixin
from .es_pagination import ElasticLimitOffsetPagination


class ElasticAPIView(views.APIView):
    """Elasticsearch base API view class."""
    es_client = None
    es_model = None
    es_filter_backends = (ElasticSearchFilter,)

    schema = EsAutoSchema()

    def get_es_search_fields(self):
        """
        Return field or fields used for search.
        The return value must be an iterable.
        """
        return getattr(self, 'es_search_fields', None)

    def get_es_filter_fields(self):
        """
        Return field or fields used for search filtering.
        The return value must be an iterable.
        """
        return getattr(self, 'es_filter_fields', tuple())

    def get_es_range_filter_fields(self):
        """
        Return field or fields used for search filtering by range.
        The return value must be an iterable.
        """
        return getattr(self, 'es_filter_fields', tuple())

    def get_es_ordering_fields(self):
        """
        Return field or fields used for search ordering.
        The return value must be an iterable.
        """
        return getattr(self, 'es_ordering_fields', None)

    def get_es_excludes_fields(self):
        """
        Return field or fields excluded from search results.
        The return value must be an iterable.
        """
        return getattr(self, 'es_excludes_fields', None)

    def get_es_client(self):
        """
        You may want to override this if you need to provide different
        Elasticsearch.client depending on the incoming request.
        """
        assert self.es_client is not None, (
            "'%s' should either include a `queryset` attribute, "
            "or override the `get_queryset()` method."
            % self.__class__.__name__
        )
        es_client = self.es_client
        if not isinstance(es_client, Elasticsearch):
            raise ValueError("Incorrect value es_client")
        return es_client

    def filter_search(self, search):
        for backend in list(self.es_filter_backends):
            search = backend().filter_search(self.request, search, self)
        return search

    def excludes_respond_fields(self, search):
        es_excludes_fields = self.get_es_excludes_fields()
        if es_excludes_fields:
            search = search.source(**{'excludes': es_excludes_fields})
        return search

    def get_es_search(self):
        if self.es_model is None:
            msg = "Cannot use %s on a view which does not have the 'es_model'"
            raise ImproperlyConfigured(msg % self.__class__.__name__)
        index = self.es_model()._get_index()
        es_client = self.get_es_client()
        s = Search(using=es_client, index=index, doc_type=self.es_model)
        return s

    def do_search(self):
        search = self.filter_search(self.get_es_search())
        search = self.excludes_respond_fields(search)
        return search

    def es_representation(self, iterable):
        """List of object instances."""
        return [item.to_dict() for item in iterable]

    def get_queryset(self):
        """
        Get the list of elastic items for this view.
        This must be an iterable, and all items must be represented.
        """
        search = self.do_search()
        return self.es_representation(search)


class ListElasticAPIView(ListElasticMixin, ElasticAPIView):
    """Concrete view for listing a queryset."""
    es_pagination_class = ElasticLimitOffsetPagination

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from functools import reduce
import re

from django.utils import six
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from elasticsearch_dsl import Q

from rest_framework import filters
from rest_framework.settings import api_settings

from .es_validators import field_validator

try:
    from rest_framework.compat import coreapi, coreschema
except ImportError:
    coreapi = coreschema = None


ORDER_PATTERN = re.compile(r'(\?|[-+])?([.\w]+$)')


class ESFieldFilter(object):
    def __init__(self, label, name=None, description=None):
        self.label = label
        self._name = name
        # API docs filter description
        self.description = description

    @property
    def name(self):
        if self._name is None:
            return self.label
        return self._name


class BaseEsFilterBackend(object):
    """
    A base class from Elastycsearch filter backend classes.
    """

    def filter_search(self, request, search, view):
        """
        Return a filtered search.
        """
        raise NotImplementedError(".filter_search() must be overridden.")

    def get_schema_fields(self, view):
        assert coreapi is not None, 'coreapi must be installed to use `get_schema_fields()`'
        assert coreschema is not None, 'coreschema must be installed to use `get_schema_fields()`'
        return []


class ElasticOrderingFilter(filters.OrderingFilter, BaseEsFilterBackend):

    def get_es_ordering_fields(self, view):
        ordering = view.get_es_ordering_fields()
        if isinstance(ordering, six.string_types):
            return (ordering,)
        return ordering

    @staticmethod
    def validation(field):
        return (field, field) if isinstance(field, six.string_types) else field

    def get_valid_fields(self, queryset, view, context={}):
        fields = self.get_es_ordering_fields(view)
        if not fields:
            return self.get_default_valid_fields(queryset, view)
        return [self.validation(field) for field in fields]

    def remove_invalid_fields(self, queryset, fields, view, request=None):
        """Remove not allowed ordering field."""
        ordering_fields = list()
        valid_fields = self.get_valid_fields(queryset,
                                             view,
                                             {'request': request})
        valid_fields = {field[1]: field[0] for field in valid_fields}
        for term in fields:
            res = ORDER_PATTERN.match(term)
            if not res:
                continue
            order, field = res.groups()
            match_field = valid_fields.get(field)
            if match_field and isinstance(match_field, six.string_types):
                ordering_fields.append((order or '') + match_field)
        return ordering_fields

    def filter_search(self, request, search, view):
        ordering = self.get_ordering(request, search, view)
        if ordering:
            search = search.sort(*ordering)
        return search


class ElasticFieldsFilter(BaseEsFilterBackend):
    filter_description = _('A filter term.')

    def get_es_filter_fields(self, view):
        return view.get_es_filter_fields()

    def filter_search(self, request, search, view):
        es_model = getattr(view, 'es_model', None)
        for item in self.get_es_filter_fields(view):
            try:
                field = reduce(lambda d, key: d[key] if d else None,
                               item.name.split('.'), es_model._doc_type.mapping)
            except KeyError:
                # Incorrect field
                continue
            args = request.query_params.get(item.label, '')
            data = [field_validator.validate(field.name, value.strip())
                    for value in args.split(',')]
            # Remove empty string and None values
            data = [value for value in data if value not in (None, '')]
            if data:
                search = search.filter('terms', **{item.name: data})
        return search

    def get_schema_fields(self, view):
        assert coreapi is not None, 'coreapi must be installed to use `get_schema_fields()`'
        assert coreschema is not None, 'coreschema must be installed to use `get_schema_fields()`'
        fields = []
        for item in self.get_es_filter_fields(view):
            field = coreapi.Field(
                name=item.label,
                required=False,
                location='query',
                schema=coreschema.String(
                    title=force_text(item.label),
                    description=force_text(item.description or self.filter_description)
                )
            )
            fields.append(field)

        return fields


class ElasticFieldsRangeFilter(ElasticFieldsFilter):

    def get_es_filter_fields(self, view):
        return view.get_es_range_filter_fields()

    def filter_search(self, request, search, view):
        es_model = getattr(view, 'es_model', None)
        for item in self.get_es_filter_fields(view):
            try:
                field = reduce(lambda d, key: d[key] if d else None,
                               item.name.split('.'), es_model._doc_type.mapping)
            except KeyError:
                # Incorrect field
                continue

            from_arg_name = request.query_params.get('from_' + item.label, '')
            to_arg_name = request.query_params.get('to_' + item.label, '')

            from_arg = field_validator.validate(field.name, from_arg_name.strip())
            to_arg = field_validator.validate(field.name, to_arg_name.strip())

            options = {}

            if from_arg:
                options['gte'] = from_arg

            if to_arg:
                options['lte'] = to_arg

            if options:
                search = search.filter('range', **{item.name: options})

        return search


class ElasticSearchFilter(BaseEsFilterBackend):
    search_param = api_settings.SEARCH_PARAM
    search_should_match = '75%'
    search_title = _('Search')
    search_description = _('A search term.')

    def get_search_query(self, request):
        """
        Search terms are set by a ?search=... query parameter,
        and may be comma and/or whitespace delimited.
        """
        return request.query_params.get(self.search_param, '')

    def get_es_query(self, s_query, s_fields, **kwargs):
        """ Create and return elasticsearch Query.

        You could overload this method for creating your custom
        search Query object.

        Arguments:
            s_query: request search param
            s_fields: search fields

        Keyword arguments:
            request: request object
            view: view object
        """
        return Q("multi_match", query=s_query, fields=s_fields)

    def filter_search(self, request, search, view):
        s_query = self.get_search_query(request)
        s_fields = view.get_es_search_fields()
        if not s_query or not s_fields:
            return search

        q = self.get_es_query(s_query, s_fields, request=request, view=view)
        search = search.query(q)
        search.query.minimum_should_match = self.search_should_match
        return search

    def get_schema_fields(self, view):
        assert coreapi is not None, 'coreapi must be installed to use `get_schema_fields()`'
        assert coreschema is not None, 'coreschema must be installed to use `get_schema_fields()`'
        return [
            coreapi.Field(
                name=self.search_param,
                required=False,
                location='query',
                schema=coreschema.String(
                    title=force_text(self.search_title),
                    description=force_text(self.search_description)
                )
            )
        ]
